import torch
import torch.nn as nn
from vietvocoder.model.layers import Flow

class WaveGlow(nn.Module):
    def __init__(self, n_mel_channels, n_flows, n_group, n_early_every, n_early_size, WaveNet_config):
        super(WaveGlow, self).__init__()
        
        assert(n_group % 2 == 0)
        
        self.n_mel_channels = n_mel_channels
        self.n_flows = n_flows
        self.n_group = n_group
        self.n_early_every = n_early_every
        self.n_early_size = n_early_size
        self.WaveNet_config = WaveNet_config
        
        self.upsampling_spect = nn.ConvTranspose1d(n_mel_channels, n_mel_channels, 1024,
                                                  stride=256 #hop_length
                                                  )
        
        self.flows = nn.ModuleList()
        
        n_WN_channels = n_group//2
        n_audio_channels = n_group
        
        for i in range(self.n_flows):
            if i % self.n_early_every == 0 and i > 0:
                n_WN_channels = n_WN_channels - self.n_early_size//2
                n_audio_channels = n_audio_channels - self.n_early_size
                
            flow = Flow(n_audio_channels, n_WN_channels, n_mel_channels*n_group, WaveNet_config)
            self.flows.append(flow)
            
        self.n_audio_channels = n_audio_channels
        
    def forward(self, x):
        spect, audio = x
        
        spect = self.upsampling_spect(spect)
        assert(spect.size(2) >= audio.size(1))
        if spect.size(2) > audio.size(1):
            spect = spect[:,:,:audio.size(1)]
        
        spect = spect.unfold(2, self.n_group, self.n_group).permute(0, 2, 1, 3)
        spect = spect.flatten(-2).permute(0, 2, 1)
        
        audio = audio.unfold(1, self.n_group, self.n_group) # squeeze operation Bx2000x8
        audio = audio.permute(0, 2, 1)
       
        log_det_W_list = []
        log_s_list = []
        output = []
        
        for i in range(self.n_flows):
            if i % self.n_early_every == 0 and i > 0:
                early_output = audio[:, :self.n_early_size]
                audio = audio[:,self.n_early_size:]
                
                output.append(early_output)
            
            audio, log_det_W, log_s = self.flows[i]((spect, audio))
            
            log_det_W_list.append(log_det_W)
            log_s_list.append(log_s)
        
        output.append(audio)
        
        z = torch.cat(output, 1)
        
        return z, log_det_W_list, log_s_list
    
    def infer(self, spect, sigma=1.0):
        device = spect.device
        
        with torch.no_grad():
            spect = self.upsampling_spect(spect)
        
        time_cutoff = self.upsampling_spect.kernel_size[0] - self.upsampling_spect.stride[0]
        spect = spect[:, :, :-time_cutoff]
        
        spect = spect.unfold(2, self.n_group, self.n_group).permute(0, 2, 1, 3)
        spect = spect.flatten(-2).permute(0, 2, 1)
        
        z = torch.FloatTensor(spect.size(0), self.n_audio_channels, spect.size(2)).normal_()
        z = z.to(device)
        z = z*sigma
        
        for i in reversed(range(self.n_flows)):
            z = self.flows[i].invert((spect, z))
            
            if i % self.n_early_every == 0 and i > 0:
                z_early = torch.FloatTensor(spect.size(0), self.n_early_size, spect.size(2)).normal_()
                z_early = z_early.to(device)
                z = torch.cat((z_early*sigma, z), 1)
        
        audio = z.permute(0, 2, 1).flatten(-2)
        
        return audio



import os


class Logger():
    def __init__(self, fname):
        path, _ = os.path.split(fname)
        os.makedirs(path, exist_ok=True)

        self.logger = open(fname, 'w')

    def log(self, string):
        self.logger.write(string+'\n')
        self.logger.flush()
        print(string)

    def __call__(self, string):
        self.log(string)

    def close(self):
        self.logger.close()



import torch
from torch.utils.data import DataLoader
import os
import numpy as np
import soundfile

from vietvocoder.loader import MelSample
from vietvocoder.model.waveglow import WaveGlow
from vietvocoder.model.layers import WaveGlowLoss
from vietvocoder.utils.logger import Logger
from vietvocoder.utils.download_model import download_weights

class Trainer():
    def __init__(self, config):
        
        data_config = config['data_config']
        train_config = config['train_config']
        waveglow_config = config['waveglow_config']

        trainset = MelSample(train_config['train_files'], **data_config)
        evalset = MelSample(train_config['test_files'], **{**data_config, 'segment_length':64000})
        
        
        batch_size = train_config['batch_size']
        workers = train_config['load_workers']
        self.log_dir = train_config['log_dir']
        self.device = train_config['device']
        self.num_iters = train_config['num_iters']
        self.eval_steps = train_config['eval_steps']
        self.sampling_rate = data_config['sampling_rate']

        self.train_loader = DataLoader(trainset, num_workers=workers, 
                                shuffle=True,
                                batch_size=batch_size,
                                pin_memory=False,
                                drop_last=True)

        self.eval_loader = DataLoader(evalset, num_workers=workers, 
                                shuffle=False,
                                batch_size=batch_size,
                                pin_memory=False,
                                drop_last=True)

        self.train_iter = iter(self.train_loader)

        self.sample_eval_mel, _ = next(iter(self.eval_loader))

        self.model = WaveGlow(**waveglow_config)

        if self.device.startswith('cuda'):
            self.model = self.model.to(self.device)
            self.sample_eval_mel = self.sample_eval_mel.to(self.device)

        self.criterion = WaveGlowLoss()
        self.optimizer = torch.optim.Adam(self.model.parameters(), lr=train_config['learning_rate'])
        
        pretrained = train_config['pretrained_url']
        if pretrained != None:
            pretrained_weights_path = download_weights(pretrained, md5=train_config['pretrained_md5'])
            self.model.load_state_dict(torch.load(pretrained_weights_path))

        if self.log_dir != None:
            log_name = os.path.join(self.log_dir, 'log.txt')
            self.log = Logger(log_name)
        
        self.best_eval_loss = 99999
        self.current_iter = 0
        
    def train(self):
        for i in range(self.current_iter, self.num_iters):
            self.current_iter = i
            try:
                mel, audio = next(self.train_iter)
            except StopIteration:
                self.train_iter = iter(self.train_loader)
                mel, audio = next(self.train_iter)
            
            loss = self.train_step(mel, audio)
            self.log('iter: {} - train_loss: {}'.format(i, loss))

            if i % self.eval_steps==0:
                eval_loss = self.evaluate()

                if self.log_dir:
                    self.sample()
                if eval_loss < self.best_eval_loss:
                    self.best_eval_loss = eval_loss
                    self.save_checkpoint()

                self.log('iter: {} - valid_loss: {}'.format(i, eval_loss))

    def save_checkpoint(self):
        state = {
	    'iter': self.current_iter,
            'best_eval_loss':self.best_eval_loss,
	    'state_dict': self.model.state_dict(),
	    'optimizer': self.optimizer.state_dict(),
	}

        torch.save(state,os.path.join(self.log_dir, 'checkpoint.pt'))
        torch.save(self.model.state_dict(), os.path.join(self.log_dir, 'model.pt'))

    def load_checkpoint(self):
        state = torch.load(os.path.join(self.log_dir, 'checkpoint.pt'))
        self.current_iter = state['iter']
        self.best_eval_loss = state['best_eval_loss']
        self.model.load_state_dict(state['state_dict'])
        self.optimizer.load_state_dict(state['optimizer'])
        
        print('continue train from the lastest checkpont. current_iter: {}, best_eval_loss: {}'.format(self.current_iter, self.best_eval_loss))

    def train_step(self, mel, audio):
        self.model.train()
        self.model.zero_grad()
    
        mel = mel.to(self.device)
        audio = audio.to(self.device)

        outputs = self.model((mel, audio))

        loss = self.criterion(outputs)

        loss.backward()

        self.optimizer.step()
        
        return loss.item()

    def evaluate(self):
        self.model.eval()
        validation_losses = []
        for batch in self.eval_loader:
            mel, audio = batch
            mel = mel.to(self.device)
            audio = audio.to(self.device)

            with torch.no_grad():
                outputs = self.model((mel, audio))

            loss = self.criterion(outputs) 
            validation_losses.append(loss.item())
        
        return np.mean(validation_losses)

    def sample(self, sigma=1.0):
        sample_audios = self.model.infer(self.sample_eval_mel, sigma=sigma).cpu().numpy()

        for k, sample_audio in enumerate(sample_audios):
            soundfile.write('{}/{:03d}_{:06d}.wav'.format(self.log_dir, k, self.current_iter), sample_audio, self.sampling_rate)



import torch
import torch.nn as nn

from scipy.signal import get_window
from librosa.util import pad_center, tiny
from librosa.filters import mel as librosa_mel_fn

def dynamic_range_compression(x, C=1, clip_val=1e-5):
    """
    PARAMS
    ------
    C: compression factor
    """
    return torch.log(torch.clamp(x, min=clip_val) * C)


def dynamic_range_decompression(x, C=1):
    """
    PARAMS
    ------
    C: compression factor used to compress
    """
    return torch.exp(x) / C

class STFT(nn.Module):
    def __init__(self, filter_length=1024, hop_length=256, win_length=1024):
        super(STFT, self).__init__()
        
        self.hop_length = hop_length
        self.win_length = win_length
        self.filter_length = filter_length
        
        fft_window = get_window('hann', win_length, fftbins=True)
        fft_window = pad_center(fft_window, filter_length)
        fft_window = torch.from_numpy(fft_window).float()
        self.register_buffer('fft_window', fft_window)
    
    def transform(self, y):
        data = torch.stft(y, n_fft=self.filter_length, hop_length=self.hop_length, 
                                win_length=self.win_length, window=self.fft_window, return_complex=True)
        magnitude = torch.sqrt(data.real**2 + data.imag**2)
        phase = torch.atan2(data.imag, data.real)
        
        return magnitude, phase
    
    def inverse(self, magnitude, phase):

        recombine_magnitude_phase = torch.complex(magnitude*torch.cos(phase), magnitude*torch.sin(phase))
        
        y = torch.istft(recombine_magnitude_phase, n_fft=self.filter_length, hop_length=self.hop_length, 
                    win_length=self.win_length, window=self.fft_window)
        
        return y

class TacotronSTFT(nn.Module):
    def __init__(self, filter_length=1024, hop_length=256, win_length=1024,
                 n_mel_channels=80, sampling_rate=22050, mel_fmin=0.0,
                 mel_fmax=8000.0):
        super(TacotronSTFT, self).__init__()
        
        self.n_mel_channels = n_mel_channels
        self.sampling_rate = sampling_rate
        self.hop_length = hop_length
        self.win_length = win_length
        self.filter_length = filter_length
        
        mel_basis = librosa_mel_fn(
            sampling_rate, filter_length, n_mel_channels, mel_fmin, mel_fmax)
        mel_basis = torch.from_numpy(mel_basis).float()
        self.register_buffer('mel_basis', mel_basis)
        
        self.stft = STFT(filter_length, hop_length, win_length)
    
    def spectral_normalize(self, magnitudes):
        output = dynamic_range_compression(magnitudes)
        return output

    def spectral_de_normalize(self, magnitudes):
        output = dynamic_range_decompression(magnitudes)
        return output

    def mel_spectrogram(self, y):
        """Computes mel-spectrograms from a batch of waves
        PARAMS
        ------
        y: Variable(torch.FloatTensor) with shape (B, T) in range [-1, 1]
        RETURNS
        -------
        mel_output: torch.FloatTensor of shape (B, n_mel_channels, T)
        """
        assert(torch.min(y.data) >= -1)
        assert(torch.max(y.data) <= 1)
                    
        magnitudes, _ = self.stft.transform(y)
    
        mel_output = torch.matmul(self.mel_basis, magnitudes)
        mel_output = self.spectral_normalize(mel_output)
        return mel_output



import torch
from torch.utils.data import Dataset
from scipy.io import wavfile
import numpy as np

from vietvocoder.stft import TacotronSTFT

MAX_WAV_VALUE = 32768.0

def load_txt(fname):
    with open(fname, 'r') as f:
        files = f.readlines()
    
    files = [f.rstrip() for f in files]
    return files

def load_audio(fname):
    sampling_rate, audio = wavfile.read(fname)
    audio = torch.from_numpy(audio.copy()).float()
    
    return sampling_rate, audio

class MelSample(Dataset):
    def __init__(self, dataset, segment_length, filter_length, hop_length, 
                 win_length, n_mel_channels, sampling_rate, mel_fmin, mel_fmax):
        
        self.dataset = load_txt(dataset)
        np.random.shuffle(self.dataset)
        
        self.segment_length = segment_length
        self.sampling_rate = sampling_rate
        
        self.stft = TacotronSTFT(filter_length=filter_length, hop_length=hop_length, 
                                 win_length=win_length, n_mel_channels=n_mel_channels, 
                                 sampling_rate=sampling_rate, 
                                 mel_fmin=mel_fmin, mel_fmax=mel_fmax)
                
    def __len__(self):
        return len(self.dataset)
    
    def get_mel(self, audio):
        audio = audio.unsqueeze(0)
        mel = self.stft.mel_spectrogram(audio)
        mel = mel.squeeze(0)
        
        return mel
    
    def __getitem__(self, index):
        fname = self.dataset[index]
        sampling_rate, audio = load_audio(fname)

        if sampling_rate != self.sampling_rate:
            raise ValueError("Current {} SR doesn't match target {} SR".format(sampling_rate, self.sampling_rate))
        
        if len(audio) > self.segment_length:
            max_start = len(audio) - self.segment_length
            start = np.random.randint(0, max_start)          
            audio = audio[start:start+self.segment_length]
        else:
            raise ValueError('Segment length must be shorter than audio size')
            
        audio = audio/MAX_WAV_VALUE
        mel = self.get_mel(audio)
        return (mel, audio)



import argparse
import numpy as np
import soundfile
import torch

from vietvocoder.model.predictor import Predictor
from vietvocoder.utils.config import Cfg

def predict(mel_file, model_weights, out_file, device):
    mel = np.load(mel_file)
    
    config = Cfg.load_config_from_name('waveglow')
    sampling_rate = config['data_config']['sampling_rate']

    predictor = Predictor(config, model_weights, device)
    audio = predictor.predict(mel)
    soundfile.write(out_file, audio, sampling_rate)

if __name__=='__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--mel_file', required=True, help='foo help')
    parser.add_argument('--weights', required=True, help='foo help')
    parser.add_argument('--device', default='cuda:0', help='foo help')
    parser.add_argument('--out_file', default='out.wav', help='foo help')

    args = parser.parse_args()

    predict(args.mel_file, args.weights, args.out_file, args.device)



import torch
from vietvocoder.stft import STFT

class Denoiser(torch.nn.Module):
    """ Removes model bias from audio produced with waveglow """

    def __init__(self, waveglow, filter_length=1024, hop_length=256,
                 win_length=1024, mode='normal'):
        super(Denoiser, self).__init__()
        
        dtype = waveglow.upsampling_spect.weight.dtype
        self.device = waveglow.upsampling_spect.weight.device
        
        self.stft = STFT(filter_length=filter_length, hop_length=hop_length, win_length=win_length)
        
        self.stft = self.stft.to(self.device)

        if mode == 'zeros':
            mel_input = torch.zeros(
                (1, 80, 88),
                dtype=dtype,
                device=self.device)
        elif mode == 'normal':
            mel_input = torch.randn(
                (1, 80, 88),
                dtype=dtype,
                device=self.device)
        else:
            raise Exception("Mode {} if not supported".format(mode))
        
        
        with torch.no_grad():
            bias_audio = waveglow.infer(mel_input, sigma=0.0).float()
            bias_spec, _ = self.stft.transform(bias_audio)
        
        self.register_buffer('bias_spec', bias_spec[:, :, 0][:, :, None])

    def forward(self, audio, strength=0.01):
        audio = audio.to(self.device).float()
        audio_spec, audio_angles = self.stft.transform(audio)
        audio_spec_denoised = audio_spec - self.bias_spec * strength
        audio_spec_denoised = torch.clamp(audio_spec_denoised, 0.0)
        audio_denoised = self.stft.inverse(audio_spec_denoised, audio_angles)
        return audio_denoised




import torch
from vietvocoder.model.waveglow import WaveGlow
from vietvocoder.model.denoiser import Denoiser

class Predictor():
    def __init__(self, config, weight, device):
        
        data_config = config['data_config']
        waveglow_config = config['waveglow_config']
        self.device = device

        self.model = WaveGlow(**waveglow_config)
        self.model.eval()
        self.model.load_state_dict(torch.load(weight))

        if self.device.startswith('cuda'):
            self.model = self.model.to(self.device)

        self.denoiser = Denoiser(self.model, 
                data_config['filter_length'],
                data_config['hop_length'],
                data_config['win_length']
                )

        

    def predict(self, mel, sigma=1.0, denoise_strength=0.01):
        mel = torch.from_numpy(mel)
        mel = mel[None,:]
        mel = mel.to(self.device)
        
        with torch.no_grad():
            sample_audios = self.model.infer(mel, sigma=sigma)
        
        denoise_audios = self.denoiser(sample_audios, denoise_strength)
        denoise_audios = denoise_audios[0].cpu().numpy()

        return denoise_audios



import argparse
import torch

from vietvocoder.model.trainer import Trainer
from vietvocoder.utils.config import Cfg

def train(train_files, test_files, batch_size, device, resume):
    
    config = Cfg.load_config_from_name('waveglow')

    train_config = config['train_config']
    train_config['train_files'] = train_files
    train_config['test_files'] = test_files
    train_config['batch_size'] = batch_size
    train_config['device'] = device

    trainer = Trainer(config)
    if resume:
        trainer.load_checkpoint()

    trainer.train()

if __name__=='__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--train_files', required=True, help='foo help')
    parser.add_argument('--test_files', required=True, help='foo help')
    parser.add_argument('--batch_size', default=8, type=int, help='foo help')
    parser.add_argument('--device', default='cuda:0', help='foo help')
    parser.add_argument('--resume', action='store_true')
   
    args = parser.parse_args()

    train(args.train_files, args.test_files, args.batch_size, args.device, args.resume) 



import torch
import torch.nn as nn
import torch.nn.functional as F

class WaveGlowLoss(nn.Module):
    def __init__(self, sigma=1.0):
        super(WaveGlowLoss, self).__init__()
        
        self.sigma = sigma
        
    def forward(self, outputs):
        z, log_det_W_list, log_s_list = outputs
        
        log_W_total = 0
        log_s_total = 0
        for i in range(len(log_det_W_list)):
            log_W_total += torch.sum(log_det_W_list[i])
            log_s_total += torch.sum(log_s_list[i])
        
        loss = torch.sum(z*z)/(2*self.sigma*self.sigma) - log_W_total - log_s_total
        loss = loss/(z.size(0)*z.size(1)*z.size(2))
        
        return loss

class Invertible1x1Conv(nn.Module):
    def __init__(self, channel):
        super(Invertible1x1Conv, self).__init__()

        self.conv = nn.Conv1d(channel, channel, kernel_size=1, stride=1, padding=0, bias=False)

        W = torch.linalg.qr(torch.FloatTensor(channel, channel).normal_())[0]

        if torch.det(W) < 0:
            W[:, 0] = -1*W[:, 0]

        W = W.view(channel, channel, 1)

        self.conv.weight.data = W

    def forward(self, audio):
        batch_size, length = audio.size(0), audio.size(2)

        W = self.conv.weight.squeeze()

        audio = self.conv(audio)

        log_det_W = batch_size*length*torch.logdet(W)

        return audio, log_det_W

    def invert(self, z):

        W = self.conv.weight.squeeze().detach()
        W_invert = W.inverse().unsqueeze(-1)

        audio = F.conv1d(z, W_invert, bias=None, stride=1, padding=0)

        return audio

@torch.jit.script
def fused_add_tanh_sigmoid_multiply(input_a, input_b, n_channels):
    n_channels_int = n_channels[0]
    in_act = input_a + input_b
    t_act = torch.tanh(in_act[:,:n_channels_int])
    s_act = torch.sigmoid(in_act[:,n_channels_int:])
    acts = t_act*s_act
    
    return acts

class WaveNet(nn.Module):
    def __init__(self, n_audio_channels, n_mel_channels, n_layers, n_channels, kernel_size):
        super(WaveNet, self).__init__()
        
        self.n_layers = n_layers
        self.n_channels = n_channels
        
        start = nn.Conv1d(n_audio_channels, n_channels, 1)
        self.start = nn.utils.weight_norm(start, name='weight')
        
        self.end = nn.Conv1d(n_channels, 2*n_audio_channels, 1) # 2*n_audio_channels for log_s and t
        self.end.weight.data.zero_()
        self.end.bias.data.zero_()
        
        cond_layer = nn.Conv1d(n_mel_channels, 2*n_channels*n_layers, 1)
        self.cond_layer = nn.utils.weight_norm(cond_layer, name='weight')
        
        self.in_layers = nn.ModuleList()
        self.res_skip_layers = nn.ModuleList()
        
        for i in range(n_layers):
            dilation = 2**i
            padding = (kernel_size*dilation - dilation)//2
            in_layer = nn.Conv1d(n_channels, 2*n_channels, kernel_size, dilation=dilation, padding=padding)
            in_layer = nn.utils.weight_norm(in_layer, name='weight')
            self.in_layers.append(in_layer)
            
            if i < n_layers - 1:
                res_skip_channels = 2*n_channels
            else:
                res_skip_channels = n_channels
            res_skip_layer = nn.Conv1d(n_channels, res_skip_channels, 1)
            res_skip_layer = nn.utils.weight_norm(res_skip_layer, name='weight')
            self.res_skip_layers.append(res_skip_layer)
            
            
    def forward(self, x):
        spect, audio = x
        
        n_channels_tensor = torch.IntTensor([self.n_channels])
        
        audio = self.start(audio)
        output = torch.zeros_like(audio)
        
        spect = self.cond_layer(spect)
        
        for i in range(self.n_layers):
            spect_offset = i*2*self.n_channels
            y = self.in_layers[i](audio) 
            spect_y = spect[:,spect_offset:spect_offset+2*self.n_channels]
            acts = fused_add_tanh_sigmoid_multiply(y, spect_y, n_channels_tensor)
            
            res_skip_acts = self.res_skip_layers[i](acts)
            
            if i < self.n_layers - 1:
                audio = audio + res_skip_acts[:,:self.n_channels]
                output = output + res_skip_acts[:,self.n_channels:]
            else:
                output = output + res_skip_acts
        
        output = self.end(output)
        
        return output

class AffineCoupling(nn.Module):
    def __init__(self, n_audio_channels, n_mel_channels, WaveNet_config):
        super(AffineCoupling, self).__init__()
        
        self.WN = WaveNet(n_audio_channels, n_mel_channels, **WaveNet_config)
    
    def forward(self, x):
        spect, audio = x
        
        n_half = audio.size(1)//2
        
        audio_0 = audio[:,:n_half]
        audio_1 = audio[:,n_half:]
        
        output = self.WN((spect, audio_0))
        
        log_s = output[:,n_half:]
        t = output[:,:n_half]
        
        audio_1 = torch.exp(log_s)*audio_1 + t
        
        audio = torch.cat((audio_0, audio_1), 1)
        
        return audio, log_s
        
    def invert(self, y):
        spect, z = y
        n_half = z.size(1)//2
        
        z_0 = z[:,:n_half]
        z_1 = z[:,n_half:]
        
        with torch.no_grad():
            output = self.WN((spect, z_0))
        
        log_s = output[:,n_half:]
        t = output[:,:n_half]
        
        audio_1 = (z_1 - t)/torch.exp(log_s)  
        audio_0 = z_0
        
        audio = torch.cat((audio_0, audio_1), 1)
        
        return audio

class Flow(nn.Module):
    def __init__(self, n_audio_channels, n_WN_channels, n_mel_channels, WaveNet_config):
        super(Flow, self).__init__()
        
        self.convinv = Invertible1x1Conv(n_audio_channels)
        self.affine_coupling = AffineCoupling(n_WN_channels, n_mel_channels, WaveNet_config)
    
    def forward(self, x):
        spect, audio = x
        
        audio, log_det_W = self.convinv(audio)
        
        audio, log_s = self.affine_coupling((spect, audio))
        
        return audio, log_det_W, log_s
    
    def invert(self, y):
        
        audio = self.affine_coupling.invert(y)      
        audio = self.convinv.invert(audio)
        
        return audio    



import gdown
import requests
import yaml

def download_weights(id_or_url, cached=None, md5=None, quiet=False):
    if id_or_url.startswith('http'):
        url = id_or_url
    else:
        url = 'https://drive.google.com/uc?id={}'.format(id_or_url)

    return gdown.cached_download(url=url, path=cached, md5=md5, quiet=quiet)

def download_config(id):
    url = 'https://raw.githubusercontent.com/pbcquoc/config/master/vietvocoder/{}.yml'.format(id)
    r = requests.get(url)
    config = yaml.safe_load(r.text)
    return config



import yaml
from vietvocoder.utils.download_model import download_config

class Cfg(dict):
    def __init__(self, config_dict):
        super(Cfg, self).__init__(**config_dict)
        self.__dict__ = self

    @staticmethod
    def load_config_from_file(fname):
        base_config = {}
        with open(fname, encoding='utf-8') as f:
            config = yaml.safe_load(f)
        base_config.update(config)

        return Cfg(base_config)

    @staticmethod
    def load_config_from_name(name):
        config = download_config(name)

        return Cfg(config)

    def save(self, fname):
        with open(fname, 'w') as outfile:
            yaml.dump(dict(self), outfile, default_flow_style=False, allow_unicode=True)
